#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2017 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from django.conf import settings
from django import db

import multiprocessing
import psutil
import signal

from ..scripts import scheduler

from beat.core.scripts import worker


#----------------------------------------------------------


class SchedulerProcess(multiprocessing.Process):

  def __init__(self, arguments, queue=None):
    super(SchedulerProcess, self).__init__()

    if queue is None:
        self.queue = multiprocessing.Queue()
    else:
        self.queue = queue

    self.arguments = arguments


  def run(self):
    self.queue.put('STARTED')
    scheduler.main(self.arguments)


#----------------------------------------------------------


class WorkerProcess(multiprocessing.Process):

  def __init__(self, arguments, queue=None):
    super(WorkerProcess, self).__init__()

    if queue is None:
        self.queue = multiprocessing.Queue()
    else:
        self.queue = queue

    self.arguments = arguments


  def run(self):
    self.queue.put('STARTED')
    worker.main(self.arguments)


#----------------------------------------------------------


def start_scheduler(settings_module='beat.web.settings.settings', interval=5,
                    address='127.0.0.1', port=50000):
    args = [
        '--settings=%s' % str(settings_module),
        '--interval=%d' % int(interval),
        '--address=%s' % str(address),
        '--port=%d' % int(port),
    ]

    if getattr(settings, 'LOCAL_SCHEDULER_VERBOSITY', None) is not None:
        args.append(settings.LOCAL_SCHEDULER_VERBOSITY)

    db.connections.close_all()

    process = SchedulerProcess(args)
    process.start()
    process.queue.get()

    return (process.pid, process)


#----------------------------------------------------------


def start_worker(name, prefix, cache, scheduler_address, use_docker=False):
    args = [
        '--prefix=%s' % str(prefix),
        '--cache=%s' % str(cache),
        '--name=%s' % str(name),
        str(scheduler_address)
    ]

    if use_docker:
        args.insert(3, '--docker')

    if getattr(settings, 'LOCAL_SCHEDULER_VERBOSITY', None) is not None:
        args.insert(3, settings.LOCAL_SCHEDULER_VERBOSITY)

    process = WorkerProcess(args)
    process.start()
    process.queue.get()

    return (process.pid, process)


#----------------------------------------------------------


def stop_process(pid):
    if not psutil.pid_exists(pid):
        return

    process = psutil.Process(pid)

    process.send_signal(signal.SIGTERM)

    gone, alive = psutil.wait_procs([process])
